#!/usr/bin/perl

# ampare reader for AVIOSYS e-Meter 8870 with Raspberry Pi 4 type B
# by SUNAGAWA Hiroyuki / Studio JamPAck
# ver.0.15 (2020/05/05)
# 
# USAGE: ./ampread.pl [tty_device_path] [read_count] [read_interval]
# ex: ./ampread.pl /dev/ttyACM0 20 0.05

# モジュール読み込み
use Time::Local;
use Device::SerialPort;
use Time::HiRes 'sleep';

# 定義

# シリアルポートは引数1から指定する
# ただし指定しなかった場合はttyACM0とする
if($ARGV[0] =~ /^\/dev\//) {
  $SERIAL_dev = $ARGV[0];
} else {
  $SERIAL_dev = "/dev/ttyACM0";
}
# 読み取り回数は引数2から指定する
# ただし指定しなかった場合は20回とする
# なお、応答速度は $SERIAL_count * $SERIAL_int (sec) になる
if($ARGV[1] =~ /^\d+$/) {
  $SERIAL_count = $ARGV[1];
} else {
  $SERIAL_count = 20;
}
# 読み込みインターバルは引数3から指定する
# ただし指定しなかった場合は0.05秒とする
if($ARGV[2] =~ /^\d+$/) {
  $SERIAL_int = $ARGV[2];
} else {
  $SERIAL_int = 0.05;
}

# ポートオープン時の待ち時間
$SERIAL_quiet = 0.7; # sec
# 読み込みバッファサイズ
$SERIAL_bytes = 200; # char
# 標準出力などをコマンドバッファリングとする
$| = 1;
# 最大実行時間
$EXECTTL = 3; # sec
# 出力ログ
#$SERIAL_log = "/var/log/aviosys_8870_ACM0";


### 処理開始

# ポートオープン
my $objtty;
$objtty = new Device::SerialPort($SERIAL_dev, $SERIAL_quiet)
  or die "ERROR: Cannot open device $SERIAL_dev $!\n";

# ポート設定
$objtty->baudrate(19200);
$objtty->databits(8);
#$dbjtty->parity("none"); # 何故か対応していないと言われたので。。。
$objtty->stopbits(1);
$objtty->handshake("none");
$objtty->write_settings()
  or die "ERROR: Cannot setup device $SERIAL_dev $!\n";

# 送受信用変数
my $SERIAL_rx_str, $SERIAL_rx_bytes;
my $SERIAL_tx_str, $SERIAL_tx_bytes;

# 定期出力OFF
$SERIAL_tx_bytes = $objtty->write("auto off\n");

# 溜まっている受信バッファをすべて流す
($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
  or die "ERROR: Cannot read device $SERIAL_dev $!\n";
do {
  ($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
    or die "ERROR: Cannot read device $SERIAL_dev $!\n";
} until($SERIAL_rx_bytes == 0);

#my $time_start = time;
my $i;
my $time_count = 0;
my $time_error = 0;
my $amp_max = 0, $amp_min = 0, $amp_avg = 0;
my $time_start = time;

# 延々受信ループ
while(time < $time_start + $EXECTTL) {
  
  # 待ち
  sleep($SERIAL_int);
  
  # readコマンド
  $SERIAL_tx_bytes = $objtty->write("read\n");
  
  # 読み取り
  ($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
    or $time_error++;
  if($SERIAL_rx_bytes == 0) {
    $time_error++;
    next;
  }
  
  # 取得値処理
  chomp $SERIAL_rx_str;
  foreach(split(/\n/,$SERIAL_rx_str)) {
    if($_ =~ /^(\d\d\.\d)A/) {
      # 最大値更新
      if($amp_max < $1) { $amp_max = $1; }
      # 最小値更新
      if($amp_min > $1 || $amp_min == 0) { $amp_min = $1; }
      # 平均値加算
      $amp_avg += $1;
      # ループカウンタ加算
      $time_count++;
      last;
    }
  }
  if($time_count >= $SERIAL_count) {
    last;
  }
}

# ポートクローズ
$objtty->close()
  or warn "WARNING: Cannot close device $SERIAL_dev $!\n";

# 平均値計算
if($time_count > 0) {
  $amp_avg /= $time_count;
}

# 出力（csv出力）
#print time. ",$time_count,";
#printf "%1.1f,%1.1f,%1.2f",$amp_min,$amp_max,$amp_avg;
#print ",$time_error\n";
# 出力（1行1値出力）
print time. "\n$time_count\n";
printf "%1.2f\n%1.1f\n%1.1f\n%1.2f\n",$SERIAL_int,$amp_min,$amp_max,$amp_avg;
print "$time_error\n";

exit 0;

